
class KidneyController < ApplicationController

  def find
    @data = params[:data] || ""
    @operation = params[:operation] || "optimal"

    @max_cycle_size = params[:max_cycle_size] || '3'
    @max_cycle_size = Integer(@max_cycle_size)

    @altruistic_chain_length = params[:altruistic_chain_length] || '1'
    @altruistic_chain_length = Integer(@altruistic_chain_length)

    constraints = params[:constraints_string] || ""
    @constraints_string = create_constraint_config(constraints.split(':'))
    @output_format = params[:output_format] || "nhsbtv1"

    respond_to do |format|
      format.html {
        start_time = Time.now
        @exchanges = KidneyExchange::KidneyAllocator.new(KIDNEY_APP_CONFIG, KIDNEY_WEIGHT_CONFIG, KIDNEY_TEMPLATES_DIR, KIDNEY_CONSTRAINTS_DIR).find_json_custom(@data,
                                                                                                                   @operation,
                                                                                                                   @altruistic_chain_length,
                                                                                                                   @max_cycle_size,
                                                                                                                   @constraints_string,
                                                                                                                   @output_format)
        @elapsed = Time.now - start_time
        @json = JSON.parse(@exchanges)

        if(@json.key?('error'))
          @error = @json['error']
          render :action => 'error'
          return
        end

        @exchange_data = KidneyModels::ExchangeData.new(@json, @operation, @max_cycle_size)

        render
      }

      format.json { render :json => KidneyExchange::KidneyAllocator.new(KIDNEY_APP_CONFIG, KIDNEY_WEIGHT_CONFIG, KIDNEY_TEMPLATES_DIR, KIDNEY_CONSTRAINTS_DIR).find_json_custom(@data,
                                                                                                                                  @operation,
                                                                                                                                  @altruistic_chain_length,
                                                                                                                                  @max_cycle_size,
                                                                                                                                  @constraints_string,
                                                                                                                                  @output_format)}

      format.xml  { render :xml => KidneyExchange::KidneyAllocator.new(KIDNEY_APP_CONFIG, KIDNEY_WEIGHT_CONFIG, KIDNEY_TEMPLATES_DIR, KIDNEY_CONSTRAINTS_DIR).find_xml_custom(@data,
                                                                                                                                @operation,
                                                                                                                                @altruistic_chain_length,
                                                                                                                                @max_cycle_size,
                                                                                                                                @constraints_string,
                                                                                                                                @output_format)}

    end
  end

  def index
    @operations = [['optimal', 'optimal'], ['maxcard','maxcard'], ['pairs', 'pairs']]
    @chain_lengths = [['1', '1'],['2','2']]
  end

private
def create_constraint_config(constraints)
    if(constraints.blank?)
      return  '{"constraints": []}'
    end

    constraints_str = '{"constraints": ['
    constraints.each {|constraint| constraints_str += '{"name":"' + constraint + '"},'}
    constraints_str.chop!
    constraints_str += "]}"
    constraints_str
  end
end
