require 'test_helper'

class KidneyControllerTest < ActionController::TestCase
  test "find_v2 returns data for jan 09 altruistic for custom constraints" do
    test_file = File.dirname(__FILE__) + '/../test_files/jan09-altruistic.json'
    as_str = IO.read(test_file)

    get(:find_v2, {:data => as_str, :operation => 'custom',
                   :max_cycle_size => "3", :altruistic_chain_length => "1",
                   :constraints_string => "size:weight", :output_version => "nhsbtv2",
                   :format => 'json'})

    data = JSON.parse(@response.body)

    expected_descr = "(COIN) iteration to maximise the total number of transplants, \
iteration to maximise the total weight"
    
    assert_equal expected_descr, data['output']['exchange_data'][0]['description']
    assert_equal 990.24965, data['output']['exchange_data'][0]['weight']
    assert_equal 29, data['output']['exchange_data'][0]['total_transplants']
    assert_equal 7, data['output']['exchange_data'][0]['two_way_exchanges']
    assert_equal 5, data['output']['exchange_data'][0]['three_way_exchanges']
  end

  test "find_v2 returns data for jan 09 altruistic for optimal" do
    test_file = File.dirname(__FILE__) + '/../test_files/jan09-altruistic.json'
    as_str = IO.read(test_file)

    get(:find_v2, {:data => as_str, :operation => 'optimal',
                   :max_cycle_size => "3", :altruistic_chain_length => "1",
                   :constraints_string => "", :output_version => "nhsbtv1",
                   :format => 'json'})

    data = JSON.parse(@response.body)

    expected_descr = "(COIN) iteration to maximise the number of effective pairwise exchanges using lemon, \
iteration to maximise the total number of transplants, \
iteration to minimise the number of 3-way exchanges, \
iteration to maximise the total number of backarcs, \
iteration to maximise the total weight"

    assert_equal expected_descr, data['output']['exchange_data'][0]['description']
    assert_equal 884.25505, data['output']['exchange_data'][0]['weight']
    assert_equal 27, data['output']['exchange_data'][0]['total_transplants']
    assert_equal 9, data['output']['exchange_data'][0]['two_way_exchanges']
    assert_equal 3, data['output']['exchange_data'][0]['three_way_exchanges']
  end

  test "find returns data for jan 09 altruistic for operation optimal" do
    test_file = File.dirname(__FILE__) + '/../test_files/jan09-altruistic.json'
    as_str = IO.read(test_file)

    get(:find, {:data => as_str,
                :operation => 'optimal',
                :altruistic_chain_length => "1",
                :format => 'json'})

    data = JSON.parse(@response.body)

    expected_descr = "(COIN) iteration to maximise the number of effective pairwise exchanges using lemon, \
iteration to maximise the total number of transplants, \
iteration to minimise the number of 3-way exchanges, \
iteration to maximise the total number of backarcs, \
iteration to maximise the total weight"

    assert_equal expected_descr, data['output']['exchange_data'][0]['description']
    assert_equal 884.25505, data['output']['exchange_data'][0]['weight']
    assert_equal 27, data['output']['exchange_data'][0]['total_transplants']
    assert_equal 9, data['output']['exchange_data'][0]['two_way_exchanges']
    assert_equal 3, data['output']['exchange_data'][0]['three_way_exchanges']
  end

  test "find returns data for jan 09 altruistic for operation maxcard" do
    test_file = File.dirname(__FILE__) + '/../test_files/jan09-altruistic.json'
    as_str = IO.read(test_file)

    get(:find, {:data => as_str,
                :operation => 'maxcard',
                :altruistic_chain_length => "1",
                :format => 'json'})

    data = JSON.parse(@response.body)

    expected_descr = "(COIN) iteration to maximise the total number of transplants, \
iteration to maximise the total weight"

    assert_equal expected_descr, data['output']['exchange_data'][0]['description']
    assert_equal 990.24965, data['output']['exchange_data'][0]['weight']
    assert_equal 29, data['output']['exchange_data'][0]['total_transplants']
    assert_equal 7, data['output']['exchange_data'][0]['two_way_exchanges']
    assert_equal 5, data['output']['exchange_data'][0]['three_way_exchanges']
  end

  test "find returns data for jan 09 altruistic for operation pairs" do
    test_file = File.dirname(__FILE__) + '/../test_files/jan09-altruistic.json'
    as_str = IO.read(test_file)

    get(:find, {:data => as_str,
                :operation => 'pairs',
                :altruistic_chain_length => "1",
                :format => 'json'})

    data = JSON.parse(@response.body)

    expected_descr = "(COIN) iteration to maximise the total number of transplants, \
iteration to maximise the total weight"

    assert_equal expected_descr, data['output']['exchange_data'][0]['description']
    assert_equal 623.12625, data['output']['exchange_data'][0]['weight']
    assert_equal 22, data['output']['exchange_data'][0]['total_transplants']
    assert_equal 11, data['output']['exchange_data'][0]['two_way_exchanges']
    assert_equal 0, data['output']['exchange_data'][0]['three_way_exchanges']
  end

end
