module KidneyModels
  class Exchange
    attr_accessor :description, :cycles, :weight, :total_transplants, :optimal
    attr_accessor :unused_altruistics, :altruistic_totals, :altruistic_used, :total_paired
    attr_reader :effective_two_ways
  
    def initialize(description, weight, n_cycle_totals, altruistic_used, altruistic_totals, total_transplants, cycles, unused_altruistics)
      @description = description;
      @cycles = cycles
      @weight = weight || 0
      @n_cycle_totals = n_cycle_totals
      @total_transplants = total_transplants || 0
      @effective_two_ways = get_effective_two_ways
      @unused_altruistics = unused_altruistics
      @altruistic_totals = altruistic_totals
	    @altruistic_used = altruistic_used
	    @total_paired = total_transplants - total_altruistic_in_chains
    end
      
    def get_num_n_ways(n)
      if(!@n_cycle_totals.has_key?(n))
        return 0
      end
      
      @n_cycle_totals[n]
    end
    
    def get_num_alt_chain_size_n(n)
      if(!@altruistic_totals.has_key?(n))
        return 0
      end
      
      @altruistic_totals[n]    
    end

    def get_total_num_embedded_cycles(cycle_size, embedded_size)
      # Handle this the old way first, otherwise if we have a cycle of length
      # greater than 3 we need to handle this via the new format
      #
      if(cycle_size == 3 && embedded_size == 2)
        total = 0    
        @cycles.each { |cycle| total += 1 if cycle.patients.length == 3 && cycle.any_back_arcs?}        
        return total;
      end
      
      total = 0
      @cycles.each do |cycle|
         next if cycle.patients.length != cycle_size         
         next if !cycle.embedded.has_key?(embedded_size)
         total += 1 if cycle.embedded[embedded_size].length > 1
      end
      
      total
    end
          
private 
    def total_altruistic_in_chains
      total = 0;
      @altruistic_totals.each do |key,value|
        total += (key*value)
      end
      total
    end
       
    def get_effective_two_ways
      count = 0
      @cycles.each { |cycle| count += 1 if cycle.any_back_arcs?}
      count + @n_cycle_totals[2]
    end

  end
end