module KidneyModels
  class ExchangeData
    attr_accessor :error_message
  
    attr_reader :all_cycles
              
    @@wordy = ["zero", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten",
               "eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen", "eighteen",
               "nineteen", "twenty"]

    def initialize(exchange_data, operation, max_cycle_size)
      @exchange_data = exchange_data
      @operation = operation
      @max_cycle_size = max_cycle_size

      if(check_for_error(exchange_data))
        return
      end
  
      get_all_cycles
	  set_n_cycle_totals
      get_matchings
  	end
  		  
    def exchanges
      @all_exchanges
    end

    def error?
      @errored
    end

    def get_total_n_cycles_in_all_cycles(n)
      @n_cycle_totals[n]
    end
    
private
    def set_altruistic_totals(exchanges)
	  @altruistic_used = 0
      @n_altruistic_totals = {}

   	  exchanges.each do |item|
        next if(!@all_cycles[item].altruistic)
        
		@altruistic_used += 1
		
        chain_len = @all_cycles[item].patients.length
        @n_altruistic_totals[chain_len] ||= 0
        @n_altruistic_totals[chain_len] += 1
   	  end
    end
    
    def get_result_totals(data)
      result_cycles_totals = {}
      (2..@max_cycle_size).each { |n| result_cycles_totals[n] = data["#{@@wordy[n]}_way_exchanges"] }
      result_cycles_totals
    end
    
  	def set_n_cycle_totals
  	  @n_cycle_totals = {}
  	  (2..@max_cycle_size).each { |n| @n_cycle_totals[n] = get_total_num_cycles(n) }
  	end
	  
    def get_total_num_cycles(size)
	    total = 0
      @all_cycles.each_key { |group_id| total += @all_cycles[group_id].patients.size == size ? 1 : 0}
  	  total
    end
  
    def get_all_cycles
      @all_cycles = {}
      @exchange_data['output']['all_cycles'].each do |id, cycle|
  	    id_as_int = Integer(id)
        @all_cycles[id_as_int] = CycleEntry.new(id_as_int, cycle);
      end
    end
    
    def get_matchings
      @all_exchanges = [];
      for data in @exchange_data['output']['exchange_data']
        set_altruistic_totals(data['exchanges'])
        @all_exchanges << Exchange.new(data['description'], data['weight'], get_result_totals(data), @altruistic_used, @n_altruistic_totals,
                                       data['total_transplants'], get_cycle_entries(data['exchanges']), data['unused_altruistics'])
      end
    end
  
    def get_cycle_entries(exchanges)
      entries = []
      exchanges.each do |element|
        entries << @all_cycles[element]
      end
      entries
    end
      
    def check_for_error(data)
      if(data.key? 'error')
        @errored = true
        @error_message = data['error']
      end
      @errored = false
    end
  end
end